@@ -11,8 +11,10 @@
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  */
+//stype-check off
 
 #include <vulkan/vulkan_core.h>
+#include <algorithm>
 #include "../framework/layer_validation_tests.h"
 #include "../framework/pipeline_helper.h"
 #include "utils/hash_util.h"
@@ -63,11 +65,10 @@ TEST_F(VkLayerTest, UnsupportedPnextApiVersion) {
 TEST_F(VkLayerTest, VuidCheckForHashCollisions) {
     TEST_DESCRIPTION("Ensure there are no VUID hash collisions");
 
-    constexpr uint64_t num_vuids = sizeof(vuid_spec_text) / sizeof(vuid_spec_text[0]);
     std::vector<uint32_t> hashes;
-    hashes.reserve(num_vuids);
-    for (const auto &vuid_spec_text_pair : vuid_spec_text) {
-        const uint32_t hash = hash_util::VuidHash(vuid_spec_text_pair.vuid);
+    hashes.reserve(GetVuidMap().size());
+    for (const auto &vuid_spec_text_pair : GetVuidMap()) {
+        const uint32_t hash = hash_util::VuidHash(vuid_spec_text_pair.first);
         hashes.push_back(hash);
     }
     std::sort(hashes.begin(), hashes.end());
@@ -83,7 +84,7 @@ TEST_F(VkLayerTest, VuidHashStability) {
     ASSERT_TRUE(hash_util::VuidHash("VUID-RayTmaxKHR-RayTmaxKHR-04349") == 0x8e67514c);
     ASSERT_TRUE(hash_util::VuidHash("VUID-RuntimeSpirv-SubgroupUniformControlFlowKHR-06379") == 0x2f574188);
     ASSERT_TRUE(hash_util::VuidHash("VVL-DEBUG-PRINTF") == 0x4fe1fef9);
-    ASSERT_TRUE(hash_util::VuidHash("WARNING-GPU-Assisted-Validation") == 0x24b5c69f);
+    ASSERT_TRUE(hash_util::VuidHash("WARNING-Setting-Limit-Adjusted") == 0x86fe6721);
 }
 
 TEST_F(VkLayerTest, RequiredParameter) {
@@ -779,10 +780,8 @@ TEST_F(VkLayerTest, InvalidImageCreateFlagWithPhysicalDeviceCount) {
     ici.tiling = VK_IMAGE_TILING_OPTIMAL;
     ici.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
 
-    VkImageFormatProperties imageFormatProperties;
-    VkResult result =
-        vk::GetPhysicalDeviceImageFormatProperties(physical_device_group[0].physicalDevices[0], ici.format, ici.imageType,
-                                                   ici.tiling, ici.usage, ici.flags, &imageFormatProperties);
+    VkImageFormatProperties image_format_properties;
+    VkResult result = GetImageFormatProps(physical_device_group[0].physicalDevices[0], ici, image_format_properties);
     if (result == VK_ERROR_FORMAT_NOT_SUPPORTED) {
         GTEST_SKIP() << "image format is not supported";
     }
@@ -1411,6 +1410,21 @@ TEST_F(VkLayerTest, MissingExtensionPipelineCreateFlags2) {
     m_errorMonitor->VerifyFound();
 }
 
+TEST_F(VkLayerTest, OverridePipelineCreateFlags2) {
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    AddRequiredFeature(vkt::Feature::maintenance5);
+    RETURN_IF_SKIP(Init());
+
+    VkPipelineCreateFlags2CreateInfo flags2 = vku::InitStructHelper();
+    flags2.flags = 0;
+
+    CreateComputePipelineHelper pipe(*this, &flags2);
+    pipe.cp_ci_.flags = VK_PIPELINE_CREATE_2_DISABLE_OPTIMIZATION_BIT;
+    m_errorMonitor->SetDesiredWarning("WARNING-VkPipelineCreateFlags2-flags1-zero");
+    pipe.CreateComputePipeline();
+    m_errorMonitor->VerifyFound();
+}
+
 TEST_F(VkLayerTest, UnkonwnStructType) {
     AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
     RETURN_IF_SKIP(Init());
@@ -1432,4 +1446,29 @@ TEST_F(VkLayerTest, DISABLED_MultipleExtensionOrDependency) {
     m_errorMonitor->SetDesiredError("VUID-vkCreateDevice-ppEnabledExtensionNames-01387");
     RETURN_IF_SKIP(Init());
     m_errorMonitor->VerifyFound();
-}
\ No newline at end of file
+}
+
+TEST_F(VkLayerTest, CooperativeMatrixProps) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-Docs/issues/2613");
+    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
+    AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::cooperativeMatrix);
+    RETURN_IF_SKIP(Init());
+    uint32_t count = 1;
+    {
+        VkCooperativeMatrixPropertiesKHR props;
+        props.sType = VK_STRUCTURE_TYPE_APPLICATION_INFO;
+        props.pNext = nullptr;
+        m_errorMonitor->SetDesiredError("VUID-VkCooperativeMatrixPropertiesKHR-sType-sType");
+        vk::GetPhysicalDeviceCooperativeMatrixPropertiesKHR(Gpu(), &count, &props);
+        m_errorMonitor->VerifyFound();
+    }
+    {
+        VkQueueFamilyProperties2 props;
+        props.sType = VK_STRUCTURE_TYPE_APPLICATION_INFO;
+        props.pNext = nullptr;
+        m_errorMonitor->SetDesiredError("VUID-VkQueueFamilyProperties2-sType-sType");
+        vk::GetPhysicalDeviceQueueFamilyProperties2KHR(Gpu(), &count, &props);
+        m_errorMonitor->VerifyFound();
+    }
+}
