@@ -15,35 +15,7 @@
 #include "../framework/layer_validation_tests.h"
 #include "../framework/pipeline_helper.h"
 #include "../framework/shader_object_helper.h"
-
-const char *vkComponentTypeToGLSL(VkComponentTypeKHR type) {
-    switch (type) {
-        case VK_COMPONENT_TYPE_FLOAT16_KHR:
-            return "float16_t";
-        case VK_COMPONENT_TYPE_FLOAT32_KHR:
-            return "float32_t";
-        case VK_COMPONENT_TYPE_FLOAT64_KHR:
-            return "float64_t";
-        case VK_COMPONENT_TYPE_SINT8_KHR:
-            return "int8_t";
-        case VK_COMPONENT_TYPE_SINT16_KHR:
-            return "int16_t";
-        case VK_COMPONENT_TYPE_SINT32_KHR:
-            return "int32_t";
-        case VK_COMPONENT_TYPE_SINT64_KHR:
-            return "int64_t";
-        case VK_COMPONENT_TYPE_UINT8_KHR:
-            return "uint8_t";
-        case VK_COMPONENT_TYPE_UINT16_KHR:
-            return "uint16_t";
-        case VK_COMPONENT_TYPE_UINT32_KHR:
-            return "uint32_t";
-        case VK_COMPONENT_TYPE_UINT64_KHR:
-            return "uint64_t";
-        default:
-            return "unknown";
-    }
-}
+#include "cooperative_matrix_helper.h"
 
 void CooperativeMatrixTest::InitCooperativeMatrixKHR() {
     AddRequiredExtensions(VK_KHR_COOPERATIVE_MATRIX_EXTENSION_NAME);
@@ -52,157 +24,23 @@ void CooperativeMatrixTest::InitCooperativeMatrixKHR() {
     AddRequiredFeature(vkt::Feature::cooperativeMatrix);
     AddRequiredFeature(vkt::Feature::vulkanMemoryModel);
     RETURN_IF_SKIP(Init());
-    uint32_t props_count = 0;
-    vk::GetPhysicalDeviceCooperativeMatrixPropertiesKHR(Gpu(), &props_count, nullptr);
-    for (uint32_t i = 0; i < props_count; i++) {
-        coop_matrix_props.emplace_back(vku::InitStruct<VkCooperativeMatrixPropertiesKHR>());
-    }
-    vk::GetPhysicalDeviceCooperativeMatrixPropertiesKHR(Gpu(), &props_count, coop_matrix_props.data());
-
-    if (IsExtensionsEnabled(VK_NV_COOPERATIVE_MATRIX_2_EXTENSION_NAME)) {
-        props_count = 0;
-        vk::GetPhysicalDeviceCooperativeMatrixFlexibleDimensionsPropertiesNV(Gpu(), &props_count, nullptr);
-        for (uint32_t i = 0; i < props_count; i++) {
-            coop_matrix_flex_props.emplace_back(vku::InitStruct<VkCooperativeMatrixFlexibleDimensionsPropertiesNV>());
-        }
-        vk::GetPhysicalDeviceCooperativeMatrixFlexibleDimensionsPropertiesNV(Gpu(), &props_count, coop_matrix_flex_props.data());
-    }
-}
-
-bool CooperativeMatrixTest::HasValidProperty(VkScopeKHR scope, uint32_t m, uint32_t n, uint32_t k, VkComponentTypeKHR type) {
-    bool found_a = false;
-    bool found_b = false;
-    bool found_c = false;
-    bool found_r = false;
-    for (const auto &prop : coop_matrix_props) {
-        if (prop.scope == scope && prop.AType == type && prop.MSize == m && prop.KSize == k) {
-            found_a = true;
-        }
-        if (prop.scope == scope && prop.BType == type && prop.KSize == k && prop.NSize == n) {
-            found_b = true;
-        }
-        if (prop.scope == scope && prop.CType == type && prop.MSize == m && prop.NSize == n) {
-            found_c = true;
-        }
-        if (prop.scope == scope && prop.ResultType == type && prop.MSize == m && prop.NSize == n) {
-            found_r = true;
-        }
-    }
-    if (found_a && found_b && found_c && found_r) {
-        return true;
-    }
-
-    found_a = false;
-    found_b = false;
-    found_c = false;
-    found_r = false;
-    for (const auto &prop : coop_matrix_flex_props) {
-        if (prop.scope == scope && prop.AType == type && (m % prop.MGranularity) == 0 && (k % prop.KGranularity) == 0) {
-            found_a = true;
-        }
-        if (prop.scope == scope && prop.BType == type && (k % prop.KGranularity) == 0 && (n % prop.NGranularity) == 0) {
-            found_b = true;
-        }
-        if (prop.scope == scope && prop.CType == type && (m % prop.MGranularity) == 0 && (n % prop.NGranularity) == 0) {
-            found_c = true;
-        }
-        if (prop.scope == scope && prop.ResultType == type && (m % prop.MGranularity) == 0 && (n % prop.NGranularity) == 0) {
-            found_r = true;
-        }
-    }
-    if (found_a && found_b && found_c && found_r) {
-        return true;
-    }
-
-    return false;
 }
 
 class PositiveShaderCooperativeMatrix : public CooperativeMatrixTest {};
 
-TEST_F(PositiveShaderCooperativeMatrix, CooperativeMatrixNV) {
-    TEST_DESCRIPTION("Test VK_NV_cooperative_matrix.");
-
-    AddRequiredExtensions(VK_NV_COOPERATIVE_MATRIX_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    // glslang will generate OpCapability VulkanMemoryModel and need entension enabled
-    AddRequiredExtensions(VK_KHR_VULKAN_MEMORY_MODEL_EXTENSION_NAME);
-    SetTargetApiVersion(VK_API_VERSION_1_3);
-    RETURN_IF_SKIP(InitFramework());
-
-    VkPhysicalDeviceFloat16Int8FeaturesKHR float16_features = vku::InitStructHelper();
-    VkPhysicalDeviceCooperativeMatrixFeaturesNV cooperative_matrix_features = vku::InitStructHelper(&float16_features);
-    VkPhysicalDeviceVulkanMemoryModelFeaturesKHR memory_model_features = vku::InitStructHelper(&cooperative_matrix_features);
-    GetPhysicalDeviceFeatures2(memory_model_features);
-    RETURN_IF_SKIP(InitState(nullptr, &memory_model_features));
-
-    std::vector<VkDescriptorSetLayoutBinding> bindings(0);
-    const vkt::DescriptorSetLayout dsl(*m_device, bindings);
-    const vkt::PipelineLayout pl(*m_device, {&dsl});
-
-    const char *csSource = R"glsl(
-        #version 450
-        #extension GL_NV_cooperative_matrix : enable
-        #extension GL_KHR_shader_subgroup_basic : enable
-        #extension GL_KHR_memory_scope_semantics : enable
-        #extension GL_EXT_shader_explicit_arithmetic_types_float16 : enable
-        layout(local_size_x = 32) in;
-        layout(constant_id = 0) const uint C0 = 1;
-        layout(constant_id = 1) const uint C1 = 1;
-        void main() {
-           // Bad type
-           fcoopmatNV<16, gl_ScopeSubgroup, 3, 5> badSize = fcoopmatNV<16, gl_ScopeSubgroup, 3, 5>(float16_t(0.0));
-           // Not a valid multiply when C0 != C1
-           fcoopmatNV<16, gl_ScopeSubgroup, C0, C1> A;
-           fcoopmatNV<16, gl_ScopeSubgroup, C0, C1> B;
-           fcoopmatNV<16, gl_ScopeSubgroup, C0, C1> C;
-           coopMatMulAddNV(A, B, C);
-        }
-    )glsl";
-
-    const uint32_t specData[] = {
-        16,
-        8,
-    };
-    VkSpecializationMapEntry entries[] = {
-        {0, sizeof(uint32_t) * 0, sizeof(uint32_t)},
-        {1, sizeof(uint32_t) * 1, sizeof(uint32_t)},
-    };
-
-    VkSpecializationInfo specInfo = {
-        2,
-        entries,
-        sizeof(specData),
-        specData,
-    };
-
-    CreateComputePipelineHelper pipe(*this);
-    pipe.cs_ = VkShaderObj(this, csSource, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_3, SPV_SOURCE_GLSL, &specInfo);
-    m_errorMonitor->SetDesiredError("VUID-VkPipelineShaderStageCreateInfo-pSpecializationInfo-06849");
-    pipe.CreateComputePipeline();
-    m_errorMonitor->VerifyFound();
-}
-
 TEST_F(PositiveShaderCooperativeMatrix, CooperativeMatrixKHR) {
     TEST_DESCRIPTION("Test VK_KHR_cooperative_matrix.");
-
     SetTargetApiVersion(VK_API_VERSION_1_3);
-
-    AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_16BIT_STORAGE_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderFloat16);
     AddRequiredFeature(vkt::Feature::storageBuffer16BitAccess);
     RETURN_IF_SKIP(InitCooperativeMatrixKHR());
-
-    VkPhysicalDeviceCooperativeMatrixPropertiesKHR props = vku::InitStructHelper();
-    GetPhysicalDeviceProperties2(props);
-    if ((props.cooperativeMatrixSupportedStages & VK_SHADER_STAGE_COMPUTE_BIT) == 0) {
-        GTEST_SKIP() << "Compute stage is not supported";
-    }
+    CooperativeMatrixHelper helper(*this);
 
     VkCooperativeMatrixPropertiesKHR subgroup_prop = vku::InitStructHelper();
     bool found_scope_subgroup = false;
-    for (const auto &prop : coop_matrix_props) {
-        if (prop.scope == VK_SCOPE_SUBGROUP_KHR) {
+    for (const auto &prop : helper.coop_matrix_props) {
+        // We only have the 16-bit features enabled, but 32-bit also works
+        if (prop.scope == VK_SCOPE_SUBGROUP_KHR && !helper.Has8BitComponentType(prop) && !helper.Has64BitComponentType(prop)) {
             found_scope_subgroup = true;
             subgroup_prop = prop;
             break;
@@ -212,15 +50,13 @@ TEST_F(PositiveShaderCooperativeMatrix, CooperativeMatrixKHR) {
         GTEST_SKIP() << "VK_SCOPE_SUBGROUP_KHR not Found";
     }
 
-    const VkSampler *ptr = nullptr;
-    const std::vector<VkDescriptorSetLayoutBinding> bindings = {
-        {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_COMPUTE_BIT, ptr},
-        {1, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_COMPUTE_BIT, ptr},
-        {2, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_COMPUTE_BIT, ptr},
-        {3, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_COMPUTE_BIT, ptr},
-    };
-    const vkt::DescriptorSetLayout dsl(*m_device, bindings);
-    const vkt::PipelineLayout pl(*m_device, {&dsl});
+    const vkt::DescriptorSetLayout dsl(*m_device,
+                                       {
+                                           {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr},
+                                           {1, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr},
+                                           {2, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr},
+                                           {3, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr},
+                                       });
 
     std::string css = R"glsl(
          #version 450 core
@@ -256,10 +92,10 @@ TEST_F(PositiveShaderCooperativeMatrix, CooperativeMatrixKHR) {
     replace(css, "%M%", std::to_string(subgroup_prop.MSize));
     replace(css, "%N%", std::to_string(subgroup_prop.NSize));
     replace(css, "%K%", std::to_string(subgroup_prop.KSize));
-    replace(css, "%type_A%", vkComponentTypeToGLSL(subgroup_prop.AType));
-    replace(css, "%type_B%", vkComponentTypeToGLSL(subgroup_prop.BType));
-    replace(css, "%type_C%", vkComponentTypeToGLSL(subgroup_prop.CType));
-    replace(css, "%type_R%", vkComponentTypeToGLSL(subgroup_prop.ResultType));
+    replace(css, "%type_A%", helper.VkComponentTypeToGLSL(subgroup_prop.AType));
+    replace(css, "%type_B%", helper.VkComponentTypeToGLSL(subgroup_prop.BType));
+    replace(css, "%type_C%", helper.VkComponentTypeToGLSL(subgroup_prop.CType));
+    replace(css, "%type_R%", helper.VkComponentTypeToGLSL(subgroup_prop.ResultType));
 
     CreateComputePipelineHelper pipe(*this);
     pipe.cs_ = VkShaderObj(this, css.c_str(), VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_3);
@@ -271,8 +107,6 @@ TEST_F(PositiveShaderCooperativeMatrix, CooperativeMatrixKHR) {
 TEST_F(PositiveShaderCooperativeMatrix, RequiredSubgroupSize) {
     TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/9843");
     SetTargetApiVersion(VK_API_VERSION_1_3);
-    AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_16BIT_STORAGE_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderFloat16);
     AddRequiredFeature(vkt::Feature::storageBuffer16BitAccess);
     AddRequiredFeature(vkt::Feature::subgroupSizeControl);
@@ -281,10 +115,7 @@ TEST_F(PositiveShaderCooperativeMatrix, RequiredSubgroupSize) {
         GTEST_SKIP() << "This makes assumption about possible coop matrix subgroup size and support.";
     }
 
-    const std::vector<VkDescriptorSetLayoutBinding> bindings = {
-        {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr},
-    };
-    const vkt::DescriptorSetLayout dsl(*m_device, bindings);
+    const vkt::DescriptorSetLayout dsl(*m_device, {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr});
     const vkt::PipelineLayout pipeline_layout(*m_device, {&dsl});
 
     const char *cs_source = R"glsl(
@@ -339,10 +170,7 @@ TEST_F(PositiveShaderCooperativeMatrix, RequiredVulkanVersionPipeline) {
         GTEST_SKIP() << "This makes assumption about possible coop matrix subgroup size and support.";
     }
 
-    const std::vector<VkDescriptorSetLayoutBinding> bindings = {
-        {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr},
-    };
-    const vkt::DescriptorSetLayout dsl(*m_device, bindings);
+    const vkt::DescriptorSetLayout dsl(*m_device, {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr});
     const vkt::PipelineLayout pipeline_layout(*m_device, {&dsl});
 
     const char *cs_source = R"glsl(
@@ -385,11 +213,8 @@ TEST_F(PositiveShaderCooperativeMatrix, RequiredVulkanVersionShaderObject) {
         GTEST_SKIP() << "This makes assumption about possible coop matrix subgroup size and support.";
     }
 
-    const std::vector<VkDescriptorSetLayoutBinding> bindings = {
-        {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr},
-    };
-    const vkt::DescriptorSetLayout dsl(*m_device, bindings);
-    const vkt::PipelineLayout pipeline_layout(*m_device, {&dsl});
+    const vkt::DescriptorSetLayout dsl(*m_device,
+                                       {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_COMPUTE_BIT, nullptr});
 
     const char *cs_source = R"glsl(
          #version 450 core
@@ -441,9 +266,7 @@ TEST_F(PositiveShaderCooperativeMatrix, BFloat16) {
 
 TEST_F(PositiveShaderCooperativeMatrix, Float8) {
     SetTargetApiVersion(VK_API_VERSION_1_3);
-    AddRequiredExtensions(VK_KHR_SHADER_FLOAT16_INT8_EXTENSION_NAME);
     AddRequiredExtensions(VK_EXT_SHADER_FLOAT8_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_8BIT_STORAGE_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::storageBuffer8BitAccess);
     AddRequiredFeature(vkt::Feature::shaderFloat8);
     AddRequiredFeature(vkt::Feature::shaderFloat8CooperativeMatrix);
@@ -465,4 +288,53 @@ TEST_F(PositiveShaderCooperativeMatrix, Float8) {
     CreateComputePipelineHelper pipe(*this);
     pipe.cs_ = VkShaderObj(this, cs_source, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_1);
     pipe.CreateComputePipeline();
+}
+
+TEST_F(PositiveShaderCooperativeMatrix, Int8) {
+    SetTargetApiVersion(VK_API_VERSION_1_3);
+    AddRequiredFeature(vkt::Feature::shaderInt8);
+    AddRequiredFeature(vkt::Feature::storageBuffer8BitAccess);
+    RETURN_IF_SKIP(InitCooperativeMatrixKHR());
+    CooperativeMatrixHelper helper(*this);
+    if (!helper.Has16x16UintProperty()) {
+        GTEST_SKIP() << "desired VkCooperativeMatrixPropertiesKHR not found";
+    }
+
+    OneOffDescriptorSet descriptor_set(m_device, {{0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr},
+                                                  {1, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr},
+                                                  {2, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr},
+                                                  {3, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr}});
+    vkt::PipelineLayout pl(*m_device, {&descriptor_set.layout_});
+
+    std::string css = R"glsl(
+         #version 450 core
+         #pragma use_vulkan_memory_model
+         #extension GL_KHR_shader_subgroup_basic : enable
+         #extension GL_KHR_memory_scope_semantics : enable
+         #extension GL_KHR_cooperative_matrix : enable
+         #extension GL_EXT_shader_explicit_arithmetic_types : enable
+         #extension GL_EXT_shader_explicit_arithmetic_types_int8 : enable
+         layout(local_size_x = 64) in;
+         layout(set=0, binding=0) coherent buffer InputA { uint8_t x[]; } inputA;
+         layout(set=0, binding=1) coherent buffer InputB { uint8_t x[]; } inputB;
+         layout(set=0, binding=2) coherent buffer InputC { uint32_t x[]; } inputC;
+         layout(set=0, binding=3) coherent buffer Output { uint32_t x[]; } outputO;
+         coopmat<uint8_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseA> matA;
+         coopmat<uint8_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseB> matB;
+         coopmat<uint32_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseAccumulator> matC;
+         coopmat<uint32_t, gl_ScopeSubgroup, 16, 16, gl_MatrixUseAccumulator> matO;
+         void main() {
+             coopMatLoad(matA, inputA.x, 0, 16, gl_CooperativeMatrixLayoutRowMajor);
+             coopMatLoad(matB, inputB.x, 0, 16, gl_CooperativeMatrixLayoutRowMajor);
+             coopMatLoad(matC, inputC.x, 0, 16, gl_CooperativeMatrixLayoutRowMajor);
+             matO = coopMatMulAdd(matA, matB, matC);
+             coopMatStore(matO, outputO.x, 0, 16, gl_CooperativeMatrixLayoutRowMajor);
+         }
+    )glsl";
+
+    CreateComputePipelineHelper pipe(*this);
+    pipe.cs_ = VkShaderObj(this, css.c_str(), VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_3);
+    pipe.cp_ci_.layout = pl;
+    pipe.CreateComputePipeline();
+    m_errorMonitor->VerifyFound();
 }
\ No newline at end of file
